
    (*********************************************************************
     *  MP3Decoder.pas                                                   *
     *                                                                   *
     *  author    : Milenko Mitrovic                                     *
     *  email     : dcoder@dsp-worx.de                                   *
     *  web       : http://dsp-worx.de                                   *
     *  date      : 31-10-2003                                           *
     *  version   : 1.30                                                 *
     *                                                                   *
     *  The contents of this file are used with permission, subject to   *
     *  the Mozilla Public License Version 1.1 (the "License"); you may  *
     *  not use this file except in compliance with the License. You may *
     *  obtain a copy of the License at                                  *
     *  http://www.mozilla.org/MPL/MPL-1.1.html                          *
     *                                                                   *
     *  Software distributed under the License is distributed on an      *
     *  "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or   *
     *  implied. See the License for the specific language governing     *
     *  rights and limitations under the License.                        *
     *                                                                   *
     *  (C) 2003 Milenko Mitrovic <dcoder@dsp-worx.de>                   *
     *                                                                   *
     *********************************************************************)

unit MP3Decoder;

interface

uses
  Windows, SysUtils, BaseClass;

const
  BASS_STREAM_DECODE      = $200000;
  BASS_UNICODE            = $80000000;
  BASS_SAMPLE_8BITS       = 1;
  BASS_SAMPLE_MONO        = 2;

type
  BASS_CHANNELINFO = record
    freq: DWORD;
    chans: DWORD;
    flags: DWORD;
    ctype: DWORD;
  	origres: DWORD;
  end;

  TMP3Decoder = class
  private
    BASS_Init : function (device: Integer; freq, flags: DWORD; win: HWND; clsid: PGUID): BOOL; stdcall;
    BASS_StreamCreateFile : function (mem: BOOL; f: Pointer; offset, length, flags: DWORD): DWORD; stdcall;
    BASS_StreamCreateURL : function (URL:PChar; offset:DWORD; flags:DWORD; proc:Pointer; user:DWORD):DWORD; stdcall;
    BASS_StreamFree : procedure (handle: DWORD); stdcall;
    BASS_Free : function : BOOL; stdcall;
    BASS_ChannelGetInfo : function (handle: DWORD; var info:BASS_CHANNELINFO):BOOL;stdcall;
    BASS_ChannelGetAttributes : function (handle: DWORD; var freq, volume: DWORD; var pan: Integer): BOOL; stdcall;
    BASS_StreamGetLength : function (handle: DWORD): Int64; stdcall;
    BASS_ChannelSetPosition : function (handle: DWORD; pos: Int64): BOOL; stdcall;
    BASS_ChannelGetData : function (handle: DWORD; buffer: Pointer; length: DWORD): DWORD; stdcall;
    fLib : hWnd;
    fDuration : Int64;
    fBitsPerSample : integer;
    fBytesPerSample : integer;
    fFrequency : integer;
    fChannels : integer;
    fChan : DWORD;
    procedure SetPosition(Pos : int64);
    procedure GetStreamInfos;
    function InitBass : Boolean;
    procedure FreeBass;
  public
    constructor Create(Filename : PWCHAR; out hr : HRESULT);
    destructor Destroy; override;
    function GetBuffer(Buffer: Pointer; Blocks: Integer) : integer;
  published
    property Duration : Int64 read fDuration;
    property BitsPerSample : integer read fBitsPerSample;
    property BytesPerSample : integer read fBytesPerSample;
    property Frequency : integer read fFrequency;
    property Channels : integer read fChannels;
    property Position : int64 write SetPosition;
  end;

implementation

uses
  MP3Filter;

constructor TMP3Decoder.Create(Filename : PWCHAR; out hr : HRESULT);
var
  Name : String;
begin
  inherited Create;
  Name := Filename;
  if not InitBass then
  begin
    hr := E_FAIL;
    Exit;
  end;

  if (Pos(lowercase(Filename),'http://') > 0) or (Pos(lowercase(Filename),'ftp://') > 0) then
  begin
    fChan := BASS_StreamCreateURL(PChar(Name), 0, BASS_STREAM_DECODE,nil,0);
  end else
  begin
    if Win32Platform = VER_PLATFORM_WIN32_NT
      then fChan := BASS_StreamCreateFile(FALSE, FileName, 0, 0, BASS_STREAM_DECODE or BASS_UNICODE)
      else fChan := BASS_StreamCreateFile(FALSE, PChar(String(FileName)), 0, 0, BASS_STREAM_DECODE);
  end;

  if (fChan = 0) then
  begin
    hr := E_FAIL;
    Exit;
  end else
  begin
    hr := S_OK;
    GetStreamInfos;
  end;
end;

destructor TMP3Decoder.Destroy;
begin
  FreeBass;
  inherited Destroy;
end;

procedure TMP3Decoder.GetStreamInfos;
var
  lInfo : BASS_CHANNELINFO;
  lFrequency : DWORD;
  lVolume : DWORD;
  lPan : integer;
begin
  BASS_ChannelGetInfo(fChan,lInfo);
  fChannels := lInfo.chans;
  if (lInfo.Flags and BASS_SAMPLE_8BITS = BASS_SAMPLE_8BITS)
    then fBitsPerSample := 8
    else fBitsPerSample := 16;
  fBytesPerSample := fBitsPerSample div 8;
	BASS_ChannelGetAttributes(fChan, lFrequency, lVolume, lPan);
  fFrequency := lFrequency;
  fDuration := BASS_StreamGetLength(fChan);
end;

procedure TMP3Decoder.SetPosition(Pos : int64);
begin
  BASS_ChannelSetPosition(fChan,Pos);
end;

function TMP3Decoder.GetBuffer(Buffer: Pointer; Blocks : Integer) : integer;
begin
  Result := BASS_ChannelGetData(fChan, Buffer, Blocks);
end;

function TMP3Decoder.InitBass : Boolean;
var
  Buf : array[0..MAX_PATH -1] of Char;
  Path : String;
begin
  Result := False;
  GetModuleFileName(HInstance,Buf,MAX_PATH);
  Path := ExtractFilePath(Buf) + 'bass.dll';
  if FileExists(Path) then
  begin
    fLib := LoadLibrary(PChar(Path));
  end else
  begin
    fLib := LoadLibrary('bass.dll');
  end;
  if fLib = 0 then
  begin
    Exit;
  end;
  @BASS_Init := GetProcAddress(fLib,'BASS_Init');
  @BASS_StreamCreateFile := GetProcAddress(fLib,'BASS_StreamCreateFile');
  @BASS_StreamCreateURL := GetProcAddress(fLib,'BASS_StreamCreateURL');
  @BASS_StreamFree := GetProcAddress(fLib,'BASS_StreamFree');
  @BASS_Free := GetProcAddress(fLib,'BASS_Free');
  @BASS_ChannelGetInfo := GetProcAddress(fLib,'BASS_ChannelGetInfo');
  @BASS_ChannelGetAttributes := GetProcAddress(fLib,'BASS_ChannelGetAttributes');
  @BASS_StreamGetLength := GetProcAddress(fLib,'BASS_StreamGetLength');
  @BASS_ChannelSetPosition := GetProcAddress(fLib,'BASS_ChannelSetPosition');
  @BASS_ChannelGetData := GetProcAddress(fLib,'BASS_ChannelGetData');
  BASS_Init(0, 44100, 0, GetDesktopWindow,nil);
  Result := True;
end;

procedure TMP3Decoder.FreeBass;
begin
  if fChan <> 0 then
  begin
    BASS_StreamFree(fChan);
    fChan := 0;
  end;
  if (fLib > 0) and Assigned(Bass_Free) then
  begin
    try
      if (InstanceCount = 0) then
      begin
        BASS_Free;
      end;
    except
      // crashes mplayer2.exe ???
    end;
    FreeLibrary(fLib);
    fLib := 0;
    @BASS_Init := nil;
    @BASS_StreamCreateFile := nil;
    @BASS_StreamCreateURL := nil;
    @BASS_StreamFree := nil;
    @BASS_Free := nil;
    @BASS_ChannelGetInfo := nil;
    @BASS_ChannelGetAttributes := nil;
    @BASS_StreamGetLength := nil;
    @BASS_ChannelSetPosition := nil;
    @BASS_ChannelGetData := nil;
  end;
end;

end.
